#ifndef CONNECT_4_H
#define CONNECT_4_H

#include "BaseClasses.h"
#include "GameTree.h"
#include "Player.h"

#define CONNECT4_BOARD_WIDTH 7
#define CONNECT4_BOARD_HEIGHT 6

#define CONNECT4_FEATURE_LENGTH 8


class Connect4GameStateFeature
{
public:
	Connect4GameStateFeature(){feature.resize(CONNECT4_FEATURE_LENGTH);};
	~Connect4GameStateFeature(){feature.clear();};
	Connect4GameStateFeature(const Connect4GameStateFeature &cpy){feature = cpy.feature;};

	Connect4GameStateFeature& operator=(const Connect4GameStateFeature &rhs){feature = rhs.feature; return *this;};

	std::vector<double> feature;
	size_t length;

	enum {BLACK_DOUBLE=0, 
		RED_DOUBLE=1, 
		BLACK_TRIPLE=2, 
		RED_TRIPLE=3, 
		BLACK_QUAD=4, 
		RED_QUAD=5, 
		BLACK_CENTER=6, 
		RED_CENTER=7};
};

enum {RED = '@', BLACK = 'X', EMPTY = 'O'};

class Connect4Board
{
public:
	Connect4Board(){};
	~Connect4Board(){};
	Connect4Board(const Connect4Board &cpy){board = cpy.board;};
	Connect4Board& operator=(const Connect4Board &rhs){board = rhs.board; return *this;};
	
	std::vector<std::vector<char> > board;
};

class Connect4GameState
{
public:
	Connect4GameState();
	~Connect4GameState();
	Connect4GameState(const Connect4GameState &cpy);
	Connect4GameState& operator=(const Connect4GameState &rhs);

	void init();
	void clear(){};
	bool spawn(Connect4GameState &child, int move_idx); //get next possible move
	bool spawn(std::vector<Connect4GameState> &children); //get all possible next moves
	int endState(); //check whether it is an endState
	bool isOpening();
	void print();

	Connect4Board getBoard(){return state;};
	char currentPlay;
private:

	Connect4Board state;
	
};

class Connect4StateEvaluator
{
public:
	Connect4StateEvaluator(){};
	~Connect4StateEvaluator(){};
	Connect4StateEvaluator(const Connect4StateEvaluator &cpy){};

	double funct(Connect4GameState &state);

	void setWeights(Connect4GameStateFeature &_weights);
	Connect4GameStateFeature getWeights(){return weights;};

	double avgDistanceFromCenter(const Connect4Board &state, char target);

	int searchK(const Connect4Board &state, char target, int searchfor);
	int searchDouble(const Connect4Board &state, char target);
	int searchTriple(const Connect4Board &state, char target);
	int searchQuad(const Connect4Board &state, char target);

	#define MAX_NEIGHBORS 3
	int searchDirection(const Connect4Board &state, char target, int init_x, int init_y, int incr_x, int incr_y);

private:
	Connect4GameStateFeature weights;
};

class Connect4Game
{
public:
	Connect4Game();
	~Connect4Game();
	Connect4Game(const Connect4Game &cpy);

	Connect4Game(std::vector<game::Player<Connect4GameState, Connect4StateEvaluator>* > _players);

	void setPlayers(std::vector<game::Player<Connect4GameState, Connect4StateEvaluator>* > _players);

	int play();

private:
	std::vector<game::Player<Connect4GameState, Connect4StateEvaluator>* > players;
	Connect4GameState currentState;
};



#endif